local ButtonControl = FlexBar2.OO.Mixin
	{
		"Load", -- Called when the class gets initiated, suposed to initialize everything
		"Create", -- Called by load if the saved settings say the class is to be created; not called otherwise
		"Destroy",
		"Show",
		"Hide",
		"Lock",
		"Unlock",
		"LockIcon",
		"UnlockIcon",
		"LoadCurrentTexture", -- Find out what texture is suposed to currently be on the button and load it.
		"GetEffectiveAttribute",
		"LoadSettings", -- Read the current settings and applies each one of them in the correct order
		"SetDefaults", -- Load the default settings for this button (works as a reset function too)
		"SetPosition",
		"SetScale",
		"SetAttribute",
	};


function ButtonControl:Activate(Button)
	if(Button) then
		self:LoadSettings(Button);
	end
end


function ButtonControl:Show(save)
	self.Frame:Show();
	if(save) then
		self.Settings.Show = true;
	end
end

function ButtonControl:Hide(save)
	self.Frame:Hide();
	if(save) then
		self.Settings.Show = false;
	end
end


function ButtonControl:Lock(save)
	self:SetChecked(false, "Unlocked");
	--if(type(self.Settings) == "table" and type(self.Settings.Attributes) == "table" and type(self.Settings.Attributes.type) == "string") then
--		self.Frame:SetAttribute("type", self.Settings.Attributes.type);
	--else
		--FlexBar2:Print("Lock: Unable to restore old type, type is '", self.Settings.Attributes.type, "' and this was unexpected");
	--end
	self:UnregisterScript("OnMouseDown", "StartDragging");
	self:UnregisterScript("OnMouseUp", "StopDragging");
	self.Locked = true
	if(save) then
		self.Settings.Lock = true;
	end
end

function ButtonControl:Unlock(save)
	self:SetChecked(true, "Unlocked");
	self.Frame:SetAttribute("type", "");
	self:RegisterScript("OnMouseDown", "StartDragging");
	self:RegisterScript("OnMouseUp", "StopDragging");
	self.Locked = false
	if(save) then
		self.Settings.Lock = false;
	end
end


function ButtonControl:LockIcon(save)
	self:UnregisterScript("OnDragStart", "DragActionStart");
	if(save) then
		self.Settings.LockIcon = true;
	end
end

function ButtonControl:UnlockIcon(save)
	self:RegisterScript("OnDragStart", "DragActionStart");
	if(save) then
		self.Settings.LockIcon = false;
	end
end


function ButtonControl:LoadCurrentTexture(onlyreturn)
	--[[
		The function will first check if there is any 'forced' texture, and use that one if there is.
		Then the function will check if the Button is bound to an action id, if it is then it will use that texture.
		After that, if the button is bound to a spell, it takes that spell's texture.
		If onlyreturn is true then the texture will be returned but not set on the button
	]]
	if(self.Settings.Texture) then
		if(not onlyreturn) then self.Frame.Icon:SetTexture(self.Settings.Texture); end
		return self.Settings.Texture;
	elseif self:GetEffectiveAttribute("type") == "action" then
		if(not onlyreturn) then self.Frame.Icon:SetTexture(GetActionTexture(self:GetEffectiveAttribute("action"))); end
		return GetActionTexture(self:GetEffectiveAttribute("action"));
	elseif self:GetEffectiveAttribute("type") == "spell" and self:GetEffectiveAttribute("spell") then
		local OK, ret = pcall(FlexBar2.BabbleSpell.GetSpellIcon, self:GetEffectiveAttribute("spell")); -- We use a pcall here in case a spell is not in BabbleSpell
		if(OK) then
			if(not onlyreturn) then self.Frame.Icon:SetTexture(ret); end
			return ret;
		else
			FlexBar2:Debug("Failed to get the texture for the spell '" .. self:GetEffectiveAttribute("spell") .."' on button " .. self.Button);
			return false, ret;
		end
	else
		return nil;
	end
end

function ButtonControl:GetEffectiveAttribute(attr)
	if not self.cached_attr then self.cached_attr = {} end
	if not self.cached_attr[attr] then
		self.cached_attr[attr] = SecureButton_GetModifiedAttribute(self.Frame, attr, SecureStateChild_GetEffectiveButton(self.Frame)) or 1
	end
	return self.cached_attr[attr]
end

function ButtonControl:LoadSettings(Button)
	Button = Button or self.Button;
	if(Button == nil) then return false; end
	self.Button = Button;
	if(type(FlexBar2.db.profile.Button[Button]) ~= "table") then
		FlexBar2.db.profile.Button[Button] = {};
	end
	self.Settings = FlexBar2.db.profile.Button[Button];
	if(type(self.Settings.Attributes) == "table") then
		for n, v in pairs(self.Settings.Attributes) do
			self.Frame:SetAttribute(n, v);
		end
	end
	if(self.Settings.Show == true) then
		self:Show();
	else
		self:Hide();
	end
	if(self.Settings.Lock == true) then
		self:Lock();
	else
		self:Unlock();
	end
	if(self.Settings.LockIcon == true) then
		self:LockIcon();
	else
		self:UnlockIcon();
	end
	if(type(self.Settings.Scale) == "number") then
		self:SetScale(self.Settings.Scale);
	end
	if(type(self.Settings.X) == "number" and type(self.Settings.Y) == "number") then
		self:SetPosition(self.Settings.X, self.Settings.Y);
	end
	self:LoadCurrentTexture();
	FlexBar2:TriggerEvent("FB2_LoadSettings");
	return true;
end


function ButtonControl:SetDefaults()
	local X, Y, Attributes, Numb;
	if(self.Button == nil) then return false; end
	if(type(self.Button) == "number") then
		Numb = self.Button;
	else
		Numb = 1;
	end
	X = 100 + (38 * (Numb - ((math.ceil(Numb / 12) - 1) * 12)));
	Y = 100 + (38 * (math.ceil(Numb / 12))); -- So anoying that Y has to be negative, but nothing I can do about it as long as I anchor to TOPLEFT
	if(Numb >= 1 and Numb <= 120) then -- Only set the attributes for a button to act like an actionbutton if the button's number would be a valid action id
		Attributes =
		{
			action = Numb,
			type = "action",
		}
	end
	FlexBar2.db.profile.Button[self.Button] =  -- We set new references here, so we must watch out
	{
		Show = true,
		Lock = false,
		LockIcon = false,
		Scale = 1,
		X = X,
		Y = Y,
		Attributes = Attributes,
		TextSubs = {
			[1] = "$(name)",
			[2] = "$(key)",
			[3] = "$(itemcount)",
			[4] = "$(buttontext)",
			[5] = "$(cd)",
		}
	};
	self.Settings = FlexBar2.db.profile.Button[self.Button];
	return true;
end

function ButtonControl:SetPosition(X, Y, RelativeTo, save)
	if(type(RelativeTo) == "table" and type(RelativeTo.Frame) == "table" and type(RelativeTo.Frame.GetPoint) == "function") then
		local _, _, _, rX, rY = RelativeTo.Frame:GetPoint();
		self.Frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", rX + X, rY + Y);
	else
		self.Frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", X, -Y);
	end
	if(save) then
		self.Settings.X = X;
		self.Settings.Y = Y;
	end
end


function ButtonControl:SetScale(Scale, save)
	self.Frame:SetScale(Scale);
	if(save) then
		self.Settings.Scale = Scale;
	end
end

function ButtonControl:SetAttribute(Attribute, Value, Save)
	self.Frame:SetAttribute(Attribute,Value);
	if(Save) then
		self.Settings.Attributes[Attribute] = Value;
	end
end


FlexBar2:RegisterHandler(ButtonControl);
